/*
 * Peace.h - Peace symbol display patterns
 *
 * Design and Implementation by: Craig A. Lindley
 * Version: 1.1
 * Last Update: 03/15/2015
 */

#ifndef PEACE_H
#define PEACE_H

#include "Necklace.h"

class Peace :
  public PatternBase {

  public:

    void setup() {
      // Select our palette
      palette = RainbowColors_p;

      // Select a random color index
      colorIndex = random8();
    }

    void tearDown() {
    }

    void drawPattern() {
      // Get peace symbol color
      uint16_t color = getColorFromPalette();

      // Draw it
      drawPeaceSymbol(color);
      delay(50);
    }

  private:
    // Private data
    
    const int COLORINDEX_INCREMENT = 2;
    
    int colorIndex;

    // Current color palette
    CRGBPalette16 palette;

    // Private methods
    uint16_t getColorFromPalette();
    void drawPeaceSymbol(uint16_t color);
};

// Return a color for the the next web segment
uint16_t Peace::getColorFromPalette() {

  CRGB c = ColorFromPalette(palette, colorIndex, 255, BLEND);

  colorIndex += COLORINDEX_INCREMENT;
  colorIndex %= 256;

  return lcd.Color565(c.red, c.green, c.blue);
}

void Peace:: drawPeaceSymbol(uint16_t color) {

  // Drawn by trial and error
  // Draw outer circle
  lcd.drawCircle(MIDX, MIDY, MIDX - 1, color);
  lcd.drawCircle(MIDX, MIDY, MIDX - 7, color);

  // Draw vertical lines
  lcd.drawLine(MIDX - 3,        7, MIDX - 3,               MIDY, color);
  lcd.drawLine(MIDX - 3, MIDY + 8, MIDX - 3, DISPLAY_HEIGHT - 6, color);

  lcd.drawLine(MIDX + 3,        7, MIDX + 3,               MIDY, color);
  lcd.drawLine(MIDX + 3, MIDY + 8, MIDX + 3, DISPLAY_HEIGHT - 6, color);

  // Draw left diagonal lines
  lcd.drawLine(MIDX - 3, MIDY,            17, DISPLAY_HEIGHT - 37, color);
  lcd.drawLine(MIDX - 3, MIDY + 8,        20, DISPLAY_HEIGHT - 30, color);

  // Draw right diagonal lines
  lcd.drawLine(MIDX + 3, MIDY, DISPLAY_WIDTH - 17, DISPLAY_HEIGHT - 37, color);
  lcd.drawLine(MIDX + 3, MIDY + 8, DISPLAY_WIDTH - 19, DISPLAY_HEIGHT - 30, color);
}

#endif







