/*
 * Plasma.h - Plasma display pattern
 *
 * Concept and Implementation by: Craig A. Lindley
 * Version: 1.1
 * Last Update: 03/15/2015
 */

#ifndef PLASMA_H
#define PLASMA_H

#include "Necklace.h"

#define NUM_OF_PLASMAS  4
#define PLASMA_TYPE_0   0
#define PLASMA_TYPE_1   1
#define PLASMA_TYPE_2   2
#define PLASMA_TYPE_3   3

class Plasma :
  public PatternBase {

  public:

    void setup() {
      terminated = false;
    }

    void drawPattern() {

      if (terminated) {
        return;
      }

      lcd.clearScreen();

      int plasmaType = random16(NUM_OF_PLASMAS);
      int paletteNumber = random16(NUM_OF_PALETTES);

      // Draw the specified plasma with the specified palette
      drawPlasma(plasmaType, paletteNumber);

      delay(500);
    }

    void tearDown() {
      terminated = true;
    }

  private:

    void drawPlasma(int plasmaType, int paletteNumber);

};

void Plasma::drawPlasma(int plasmaType, int paletteNumber) {

  int colorIndex;
  float value = 0;

  // Generate specified palette
  generatePalette(paletteNumber);

  // Generate some float factors to alter plasma
  float f1 = (float) random16(2, 32);
  float f2 = (float) random16(2, 64);
  float f3 = (float) random16(2, 32);

  for (int x = 0; x < DISPLAY_WIDTH; x++) {
    for (int y = 0; y < DISPLAY_HEIGHT; y++) {

      // Determine plasma type
      switch (plasmaType) {

        case PLASMA_TYPE_0:
          value = sin(sqrt((x - MIDX) * (x - MIDX) + (y - MIDY) * (y - MIDY)) / f1);
          break;

        case PLASMA_TYPE_1:
          value = (sin(x / f1) + sin(y / f1)) / 2.0;
          break;

        case PLASMA_TYPE_2:
          value = (sin(x / f1) + sin(y / f2) + sin((x + y) / f3)) / 3.0;
          break;

        case PLASMA_TYPE_3:
          value  = sin(x / f1);
          value += sin(y / f2);
          value += sin(sqrt(((x - MIDX) * (x - MIDX)) + ((y - MIDY) * (y - MIDY))) / f3);
          value /= 3.0;
          break;
      }
      // Scale -1 ... +1 values to 0 ... 255
      value = (value * 128.0) + 128.0;
      colorIndex = ((int) value) % 256;
      lcd.drawPixel(x, y, lcd.Color565(palette[colorIndex].red, palette[colorIndex].green, palette[colorIndex].blue));
    }
  }
  delay(400);
}

#endif







