/*
 * Waves.h - Waves display patterns
 *
 * Portions of this code are adapted from Aurora: Copyright (c) 2014 Jason Coon
 *
 * Adaptation and Implementation by: Craig A. Lindley
 * Version: 1.1
 * Last Update: 03/15/2015
 */

#ifndef WAVES_H
#define WAVES_H

class Waves :
  public PatternBase {

  public:

    void setup() {

      lcd.clearScreen();

      rotation = random16(4);
      waveCount = random16(1, 3);
      createRandomPalette();
      terminated = false;
    }

    void drawPattern() {

      if (terminated) {
        return;
      }

      int n = 0;

      switch (rotation) {
        case 0:
          for (int x = 0; x < DISPLAY_WIDTH; x++) {
            n = quadwave8(x + theta) / scale;
            CRGB c = ColorFromPalette(palette, hue, 255);
            uint16_t color = lcd.Color565(c.red, c.green, c.blue);
            lcd.drawPixel(x, n, color);
            lcd.drawPixel(x, n + 1, color);
            if (waveCount == 2) {
              lcd.drawPixel(x, maxY - n, color);
              lcd.drawPixel(x, maxY - n + 1, color);
            }
          }
          break;

        case 1:
          for (int y = 0; y < DISPLAY_HEIGHT; y++) {
            n = quadwave8(y + theta) / scale;
            CRGB c = ColorFromPalette(palette, hue, 255);
            uint16_t color = lcd.Color565(c.red, c.green, c.blue);
            lcd.drawPixel(n, y, color);
            lcd.drawPixel(n + 1, y, color);
            if (waveCount == 2) {
              lcd.drawPixel(maxX - n, y, color);
              lcd.drawPixel(maxX - n + 1, y, color);
            }
          }
          break;

        case 2:
          for (int x = 0; x < DISPLAY_WIDTH; x++) {
            n = quadwave8(x - theta) / scale;
            CRGB c = ColorFromPalette(palette, hue, 255);
            uint16_t color = lcd.Color565(c.red, c.green, c.blue);
            lcd.drawPixel(x, n, color);
            lcd.drawPixel(x, n + 1, color);
            if (waveCount == 2) {
              lcd.drawPixel(x, maxY - n, color);
              lcd.drawPixel(x, maxY - n + 1, color);
            }
          }
          break;

        case 3:
          for (int y = 0; y < DISPLAY_HEIGHT; y++) {
            n = quadwave8(y - theta) / scale;
            CRGB c = ColorFromPalette(palette, hue, 255);
            uint16_t color = lcd.Color565(c.red, c.green, c.blue);
            lcd.drawPixel(n, y, color);
            lcd.drawPixel(n + 1, y, color);
            if (waveCount == 2) {
              lcd.drawPixel(maxX - n, y, color);
              lcd.drawPixel(maxX - n + 1, y, color);
            }
          }
          break;
      }

      if (thetaUpdate >= thetaUpdateFrequency) {
        thetaUpdate = 0;
        theta++;
      }
      else {
        thetaUpdate++;
      }

      if (hueUpdate >= hueUpdateFrequency) {
        hueUpdate = 0;
        hue++;
      }
      else {
        hueUpdate++;
      }

      Serial.println(hue);
      delay(5);
    }

    void tearDown() {

      terminated = true;
    }

  private:

    // Private data
    uint8_t thetaUpdate = 0;
    uint8_t thetaUpdateFrequency = 2;
    uint8_t theta = 0;

    uint8_t hueUpdate = 0;
    uint8_t hueUpdateFrequency = 15;
    uint8_t hue = 0;

    uint8_t rotation;

    int scale = 2;

    int maxX = DISPLAY_WIDTH - 1;
    int maxY = DISPLAY_HEIGHT - 1;

    int waveCount;

    // Current color palette
    CRGBPalette16 palette;

    // Private member functions
    void createRandomPalette();
};

// Create a random palette
void Waves::createRandomPalette() {

  switch (random16(8)) {
    case 0:
      palette = RainbowColors_p;
      break;
    case 1:
      palette = RainbowStripeColors_p;
      break;
    case 2:
      palette = OceanColors_p;
      break;
    case 3:
      palette = CloudColors_p;
      break;
    case 4:
      palette = ForestColors_p;
      break;
    case 5:
      palette = PartyColors_p;
      break;
    case 6:
      palette = HeatColors_p;
      break;
    case 7:
      palette = LavaColors_p;
      break;
  }
}

#endif





