/*
 * Web.h - Web display patterns
 *
 * Concept and Implementation by: Craig A. Lindley
 * Version: 1.1
 * Last Update: 03/15/2015
 */

#ifndef WEB_H
#define WEB_H

#include "Necklace.h"

class Web :
  public PatternBase {

  public:

    void setup() {
      terminated = false;

      // Create a random palette
      createRandomPalette();

      colorIndex = 0;

      even = 0;
    }

    void drawPattern() {

      uint16_t color;

      if (terminated) {
        return;
      }

      // Get a color
      color = getColorFromPalette();
      for (int x = 0; x < DISPLAY_WIDTH; x += PATTERN_INCREMENT) {
        lcd.drawLine(x, 0, MAXX, x, color);
        delay(LINE_DELAY);
      }

      // Get a color
      color = getColorFromPalette();
      for (int y = 0; y < DISPLAY_HEIGHT; y += PATTERN_INCREMENT) {
        lcd.drawLine(MAXX, y, MAXX - y, MAXY, color);
        delay(LINE_DELAY);
      }

      // Get a color
      color = getColorFromPalette();
      for (int x = MAXX; x >= 0; x -= PATTERN_INCREMENT) {
        lcd.drawLine(x, MAXY, 0, x, color);
        delay(LINE_DELAY);
      }

      // Get a color
      color = getColorFromPalette();
      for (int y = MAXY; y >= 0; y -= PATTERN_INCREMENT) {
        lcd.drawLine(0, y, MAXX - y, 0, color);
        delay(LINE_DELAY);
      }
    }

    void tearDown() {
      terminated = true;
    }

  private:
    // Private data

    const int PATTERN_INCREMENT = 8;
    const int COLORINDEX_INCREMENT = 10;
    const int LINE_DELAY = 50;

    int colorIndex;

    // Current color palette
    CRGBPalette16 palette;

    uint16_t color;
    boolean even;

    // Private member functions
    void createRandomPalette();
    uint16_t getColorFromPalette();
};

// Create/select a random palette
void Web::createRandomPalette() {

  switch (random16(2)) {
    case 0:
      palette = RainbowColors_p;
      break;
    case 1:
      palette = PartyColors_p;
      break;
  }
}

// Return a color for the the next web segment
uint16_t Web::getColorFromPalette() {

  int index;

  colorIndex += COLORINDEX_INCREMENT;
  colorIndex %= 256;

  if (even) {
    index = colorIndex;
  }
  else    {
    index = 255 - colorIndex;
  }
  even = ! even;

  CRGB c = ColorFromPalette(palette, index, 255);
  return lcd.Color565(c.red, c.green, c.blue);
}

#endif

